import sys, time, pygame, random, math, copy
from pygame.locals import *

import myDataloader, myFrames, myObjects, myLevels
import PW14_objects as customObjects
import PW14_variables as customVariables
from myConstants import *


				
class MadColorScience(myLevels.Level):				
	def __init__(self, nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers):
		myLevels.Level.__init__(self, nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)	
		self.backgroundImage=pygame.Surface(screenSize,pygame.SRCALPHA, 32).convert_alpha()
		self.backgroundImage.fill(pygame.Color('Black'))
		self.pauseScreen=None
		self.gameStarted=False
		self.countdownScreen=None
		
		self.closeBG=[]
		self.closeBG.append(self.addObject(customObjects.CloseBackGround(self.animations['screen_close.png'],self.screenSize,(0,0))))
		self.closeBG.append(self.addObject(customObjects.CloseBackGround(self.animations['screen_close.png'],self.screenSize,(self.screenSize[0],0))))
		self.farBG=[]
		self.farBG.append(self.addObject(customObjects.FarBackGround(self.animations['screen_far.png'],(self.screenSize[0]*2,self.screenSize[1]),(0,0))))
		self.farBG.append(self.addObject(customObjects.FarBackGround(self.animations['screen_far.png'],(self.screenSize[0]*2,self.screenSize[1]),(self.screenSize[0]*2,0))))
		
		self.clock=0
		self.speed=4*FPS
		
		self.bullets=[]
		self.enemies=[]
		self.checkPoint=[]
		self.lanes=[]
		self.cells=[]

		self.cellsTotalNumber=32
		self.cellsWidth=self.screenSize[0]/8
		self.cellsGapBetween=2

		
		#self.speedtext=self.addObject(myObjects.GameText('',20,(10,10),pygame.Color('Orange')))
		#self.addObject(customObjects.Light(pygame.Color('Grey'),self.you.rect.topleft))
		#self.checkPoint.append(self.addObject(customObjects.Laser(pygame.Color('White'),(10,400),(0,300))))
		
		
	def generateLanes(self):
		for lane in range(self.lanesTotalNumber):
			self.lanes.append([])
			for cell in range(-self.cellsTotalNumber/2,self.cellsTotalNumber/2):	
				#if lane == 0: print 'cell generated at',cell*(self.cellsWidth+self.cellsGapBetween),self.lanesCenterYOfMiddleOne+lane*(self.lanesGapBetween+self.lanesHeight)
				newCellCenterPosition=self.getPositionFromLaneCell(lane,cell)
				newCell=self.addObject(customObjects.ColorSquareSection(self.lanesColors[lane],lane,cell,(self.cellsWidth,self.lanesHeight),newCellCenterPosition))
				#if lane==0: print 'new cell',newCell.cell,(newCell.rect.left,newCell.rect.right),newCell.size
				#corsie oblique da implementare
				'''
				newCell=self.addObject(customObjects.ColorParallelogramSection(self.lanesColors[lane],(self.cellsWidth,self.lanesHeight),(cell*(self.cellsGapBetween+self.cellsWidth),self.lanesCenterYOfMiddleOne+lane*(self.lanesGapBetween+self.lanesHeight)-cell*self.lanesHeight*math.sin(math.radians(self.lanesAngleDegrees))),self.lanesAngleDegrees))
				'''
				self.lanes[lane].append(newCell)
			newCell=None

	def generateEnemies(self, numOfEnemies, enemyType=None):
		for e in range(numOfEnemies):
			while True:
				cellBusy=False
				enemyLane=random.randrange(self.lanesTotalNumber)
				enemyCell=random.choice([random.randrange(-self.cellsTotalNumber/2,self.enemiesNotInCellRange[0]),random.randrange(self.enemiesNotInCellRange[1],self.cellsTotalNumber/2)])
				for e in self.enemies:
					if enemyLane==e.lane and enemyCell==e.cell:
						print enemyLane,enemyCell,'is busy'
						cellBusy=True
						break
				if not cellBusy:
					break

			enemyColor=random.choice(self.enemiesColors)
			enemyCenterPosition=self.getPositionFromLaneCell(enemyLane,enemyCell)
			if enemyType==None:
				enemyType=random.choice(self.enemiesAvailable)
			enemySize=(self.cellsWidth*2,self.cellsWidth*2)
			if enemyType=='stop':
				enemySize=(self.cellsWidth*4,self.cellsWidth*4)
				enemy=self.addObject(customObjects.Enemy_Block(self.animations['obj_mine.png'],enemyColor,enemyLane,enemyCell,enemySize,enemyCenterPosition,random.choice(self.tetrisPieces)))
			elif enemyType=='move':
				enemy=self.addObject(customObjects.Enemy_Move(self.animations['obj_mecha_bug.png'],enemyColor,enemyLane,enemyCell,enemySize,enemyCenterPosition,random.choice(self.tetrisPieces)))
			elif enemyType=='shot':
				enemy=self.addObject(customObjects.Enemy_Shot(self.animations['obj_maintenance_bot.png'],enemyColor,enemyLane,enemyCell,enemySize,enemyCenterPosition,random.choice(self.tetrisPieces)))
			elif enemyType=='color':
				enemyColor=random.choice(self.enemiesColors+[self.JollyColor])
				enemy=self.addObject(customObjects.Enemy_ChangeColor(self.animations['obj_brain.png'],enemyColor,enemyLane,enemyCell,enemySize,enemyCenterPosition,random.choice(self.tetrisPieces)))
			elif enemyType=='move+shot':
				enemy=self.addObject(customObjects.Enemy_MoveShot(self.animations['obj_fake_pheega.png'],enemyColor,enemyLane,enemyCell,enemySize,enemyCenterPosition,random.choice(self.tetrisPieces)))
			elif enemyType=='move+color':
				enemyColor=random.choice(self.enemiesColors+[self.JollyColor])
				enemy=self.addObject(customObjects.Enemy_MoveChangeColor(self.animations['obj_mecha_dog.png'],enemyColor,enemyLane,enemyCell,enemySize,enemyCenterPosition,random.choice(self.tetrisPieces)))
			elif enemyType=='shot+color':
				enemySize=(self.cellsWidth*2*4/5,self.cellsWidth*2*4/5)
				enemyColor=random.choice(self.enemiesColors+[self.JollyColor])
				enemy=self.addObject(customObjects.Enemy_ShotChangeColor(self.animations['obj_trooper.png'],enemyColor,enemyLane,enemyCell,enemySize,enemyCenterPosition,random.choice(self.tetrisPieces)))
			elif enemyType=='boss':
				enemyColor=random.choice(self.enemiesColors+[self.JollyColor])
				enemy=self.addObject(customObjects.Enemy_Boss(self.animations['obj_boss.png'],enemyColor,enemyLane,enemyCell,enemySize,enemyCenterPosition,random.choice(self.tetrisPieces)))
			else:
				#to be added other types?
				enemy=self.addObject(customObjects.Enemy_Block(self.animations['obj_mine.png'],enemyColor,enemyLane,enemyCell,enemySize,enemyCenterPosition,random.choice(self.tetrisPieces)))
				
			print enemyType,'enemy generated at lane',enemyLane,'cell',enemyCell,enemyCenterPosition
			self.enemySetTileColors(enemy)
			self.enemies.append(enemy)
			enemyType=None
			enemy=None
			

		
	def generateTetrisPieces(self):
		tetrisPieces=[]
		tetrisPieces.append([[0,0],[-1,0],[-2,0],[-3,0]])
		tetrisPieces.append([[0,0],[-1,0],[-2,0],[-2,1]])
		tetrisPieces.append([[0,0],[-1,0],[-2,0],[-2,-1]])
		tetrisPieces.append([[0,0],[-1,0],[-2,0],[-1,1]])
		tetrisPieces.append([[0,0],[-1,0],[-1,-1],[0,-1]])
		tetrisPieces.append([[0,0],[-1,0],[-1,-1],[-2,-1]])
		tetrisPieces.append([[0,0],[-1,0],[-1,1],[-2,1]])	
		return tetrisPieces
		
	def getPositionFromLaneCell(self,lane,cell):
		return self.getXfromCell(cell),self.getYfromLane(lane)
	
	def getYfromLane(self,lane):
		return self.lanesCenterYOfMiddleOne+(lane-self.lanesTotalNumber/2)*(self.lanesGapBetween+self.lanesHeight)
		
	def getXfromCell(self,cell):
		return cell*(self.cellsWidth+self.cellsGapBetween)

	def setTileColors(self,lane,cell,influence,color,alpha):
		#sets tile colors according to given lane, cell, influence, color, alpha
		#print 'changing color of lane',lane,'cell',cell,'influence',influence
		for infl in influence:
			l=(lane+infl[1])%self.lanesTotalNumber
			c=(cell+infl[0])%self.cellsTotalNumber
			self.lanes[l][c-self.cellsTotalNumber/2].setColor(color,alpha)

	def restoreTileColors(self,lane,cell,influence):
		#reset tile to default color according to given lane, cell, influence
		#print 'restoring color of lane',lane,'cell',cell,'influence',influence
		for infl in influence:
			l=(lane+infl[1])%self.lanesTotalNumber
			c=(cell+infl[0])%self.cellsTotalNumber
			self.lanes[l][c-self.cellsTotalNumber/2].setColor(self.lanesColors[l],100)			
			
	def enemySetTileColors(self,enemy):
		#sets tile colors according to enemy (position, influence, color)
		self.setTileColors(enemy.lane,enemy.cell,enemy.influence,enemy.color,255)
			
	def enemyRestoreTileColors(self,enemy):
		#reset tile to default color according to enemy (position, influence) - e.g when he dies
		self.restoreTileColors(enemy.lane,enemy.cell,enemy.influence)
			
	def enemyKilled(self,enemy):
		self.enemyRestoreTileColors(enemy)
		enemy=self.delObject(enemy,[self.enemies])
		

	def checkKeyPressure(self,keyEvent):
		if self.currentLevel == MADCOLORS:
			if self.randomizeColors:
				if self.countdownScreen<>None:
					if keyEvent.type == KEYDOWN:
						if keyEvent.key == K_RETURN:
							self.countdownScreen.changeColors=True
						else:
							self.lanesColors=self.countdownScreen.randomColors[:6]
							self.JollyColor=self.countdownScreen.randomColors[6]
							self.LoserColor=self.countdownScreen.randomColors[7]
							self.initializeWithRandomColors()
							self.randomizeColors=False
							#self.delObject(self.countdownScreen)

		if self.levelPaused:
			if keyEvent.type == KEYDOWN:
				if keyEvent.key == K_ESCAPE:
					self.jumpToLevel(TITLESCREEN)
					#sys.exit('Pressed ESC')
				else:
					self.levelPaused=not self.levelPaused
					print 'Game pause',self.levelPaused
					return True
		
		if keyEvent.type == KEYDOWN:
			if keyEvent.key == K_ESCAPE:
				self.levelPaused=not self.levelPaused
				print 'Game pause',self.levelPaused
				return True
			
			#cheat/debug
			elif keyEvent.key == K_PAGEUP:
				self.jumpToLevel(self.currentLevel-1)
			elif keyEvent.key == K_PAGEDOWN:
				self.inventory['lives']=self.you.lives
				if self.currentLevel==LEVEL3:
					self.inventory['madColors']=True		
				self.jumpToLevel(self.currentLevel+1)
			#cheat/debug
			
			elif keyEvent.key == customVariables.move_up and self.you.lane>0:
				self.you.lane-=1
				self.you.status='CHANGE_LANE_UP'
				self.you.setNextAnimation('change_lane_up',-1,True)
			elif keyEvent.key == customVariables.move_down and self.you.lane<self.lanesTotalNumber-1:
				self.you.lane+=1
				self.you.status='CHANGE_LANE_DOWN'
				self.you.setNextAnimation('change_lane_down',-1,True)
			elif keyEvent.key == K_RIGHT:
				pass
			elif keyEvent.key == K_LEFT:
				pass
			else:
				for shotKeys in customVariables.shot_triple:
					if keyEvent.key in shotKeys:
						bulletColor=self.yourBulletColors[shotKeys.index(keyEvent.key)]
						bulletStartPosition=self.you.rect.right,self.getYfromLane(self.you.lane)
						self.bullets.append(self.addObject(customObjects.Bullet(bulletColor,(10,10),self.you.lane,bulletStartPosition,'you')))
						break
			
	

	def bulletOnCell(self, bullet, cell):
		if bullet.color[:3]==cell.color[:3]:#do not consider alpha
			return True
		else:
			return False
		
	def customUpdate(self):
		#countdown
		if not self.gameStarted:
			if self.countdownScreen==None:
				if self.currentLevel==MADCOLORS:
					self.countdownScreen=self.addObject(customObjects.MadColorCountDown(self.screenSize,self.animations['obj_colormix.png'], self.currentLevel, 4))
				else:
					self.countdownScreen=self.addObject(customObjects.CountDown(self.screenSize,self.animations['obj_colormix.png'], self.currentLevel, 4))
				return
			else:
				if self.currentLevel==MADCOLORS:
					if not self.randomizeColors:
						self.countdownScreen.countdown=True
						self.randomizedMadColors=self.countdownScreen.changedImage.make_surface()						
				if self.countdownScreen.start:
					self.countdownScreen=self.delObject(self.countdownScreen)
					self.gameStarted=True
			return
			
	
		#pause condfition
		if self.levelPaused:
			if self.pauseScreen==None:
				if self.currentLevel==MADCOLORS:
					self.pauseScreen=self.addObject(customObjects.PauseScreen(self.screenSize,self.animations['obj_colormix.png'], self.currentLevel,self.randomizedMadColors))
				else:
					self.pauseScreen=self.addObject(customObjects.PauseScreen(self.screenSize,self.animations['obj_colormix.png'], self.currentLevel))
			return
		else:
			self.clock+=1		
			if self.pauseScreen<>None:
				self.pauseScreen=self.delObject(self.pauseScreen)
				self.pauseScreen=None

			
		#win condition
		if len(self.enemies)==0:
			self.inventory['lives']=self.you.lives
			if self.currentLevel==LEVEL3:
				self.inventory['madColors']=True
			self.setLevelCompleted()

		#horiz lane 
		for lane in self.lanes:
			for cell in lane:
				#cell scroll
				if cell.rect.centerx>=self.getXfromCell(-self.cellsTotalNumber/2):#-self.cellsWidth:
					cell.rect.centerx-=self.speed/FPS
				#cell out of screen --> respawn
				else:
					#if lane==self.lanes[0]: print lane[0].rect.left,lane[1].rect.left,lane[2].rect.left,lane[-1].rect.left
					cell.rect.centerx=self.getXfromCell(self.cellsTotalNumber/2)#+self.cellsGapBetween#-self.cellsWidth
					#if lane==self.lanes[0]: print 'cell',cell.cell,'out of screen regenerated at',cell.rect.centerx,cell.size
		
		#corsie movimento obliquo da implementare  
		'''
		for lane in self.lanes:
			for cell in lane:
				if cell.rect.left<=-self.cellsWidth:
					cell.rect.left=(self.cellsTotalNumber-1)*self.cellsWidth
				else:
					cell.rect.left-=(self.speed/FPS)*math.cos(math.radians(self.lanesAngleDegrees))
					cell.rect.top+=(self.speed/FPS)*math.sin(math.radians(self.lanesAngleDegrees))
		'''		
		#background moving
		for cbg in self.closeBG:
			if cbg.rect.left>-cbg.size[0]:
				cbg.rect.left-=self.speed/FPS
			else:
				cbg.rect.left=cbg.size[0]-6
		if self.clock%(FPS)==0:
			for fbg in self.farBG:
				if fbg.rect.left>-2*fbg.size[0]:
					fbg.rect.left-=self.speed/FPS
				else:
					fbg.rect.left=fbg.size[0]-1
		
		#special action for selected enemies 
		if self.clock%FPS==0:
			e=random.choice(self.enemies)
			if e.rect.right>0 and e.rect.left<self.screenSize[0]:
				(lane,color)=e.specialAction([self.lanesTotalNumber,random.choice(self.enemiesColors+[self.JollyColor])])
				if lane<>None:
					self.restoreTileColors(lane,e.cell,e.influence)
					self.enemySetTileColors(e)
				if color<>None:
					self.enemySetTileColors(e)
					

		#enemies
		for e in self.enemies:
			#enemy scroll
			if e.rect.left>self.getXfromCell(-self.cellsTotalNumber/2):
				e.rect.left-=self.speed/FPS
			#enemy out of screen --> respawn
			else:
				e.rect.left=self.getXfromCell(self.cellsTotalNumber/2)


			#enemies changing lane
			if e.status=='CHANGE_LANE':
				e.rect.centery+=math.copysign(self.speed/FPS,self.getYfromLane(e.lane)-e.rect.centery)
				if abs(self.getYfromLane(e.lane)-e.rect.centery)<10:
					e.rect.centery=self.getYfromLane(e.lane)
					e.status='FLY_FW'
					

			#enemised shooting
			if e.shot:
				bulletStartPosition=(e.rect.left,self.getYfromLane(e.lane))
				if e.__class__==customObjects.Enemy_MoveShot:
					bulletColor=self.JollyColor
				else:
					bulletColor=random.choice(self.lanesColors+[self.JollyColor])
				self.bullets.append(self.addObject(customObjects.Bullet(bulletColor,(10,10),e.lane,bulletStartPosition,'enemy')))
				e.shot=False

				
			#enemies hits you --> subtract health to both
			if pygame.sprite.collide_rect(e,self.you):
				if e.lane==self.you.lane:
					killed=self.you.hit()
					#self.you.setNextAnimation('hit',1,True)
					if killed:
						self.you.death()
						self.jumpToLevel(GAMEOVER)
						break				
					killed=e.hit()
					if killed:
						e.death()
						self.enemyKilled(e)
						continue

		

		
		#you 
		if self.you.status=='CHANGE_LANE_UP':
			if self.you.flyUPSpeed<self.you.flyUPSpeedMAX:
				self.you.flyUPSpeed-=0.1
			if self.you.rect.centery<self.getYfromLane(self.you.lane):
				self.you.flyUPSpeed=0
				self.you.rect.centery=self.getYfromLane(self.you.lane)
				self.you.status='FLY_FW'
				self.you.setNextAnimation('fly',-1,True)
			
		elif self.you.status=='CHANGE_LANE_DOWN':
			if self.you.flyUPSpeed<self.you.flyUPSpeedMAX:
				self.you.flyUPSpeed+=0.1
			if self.you.rect.centery>self.getYfromLane(self.you.lane):
				self.you.flyUPSpeed=0
				self.you.rect.centery=self.getYfromLane(self.you.lane)
				self.you.status='FLY_FW'
				self.you.setNextAnimation('fly',-1,True)
				
		#bullet 
		for b in self.bullets:
			bulleHit=False
			if b.owner=='you':
				b.rect.left+=self.speed/FPS
			else:
				#print 'bullet in',b.lane,b.rect.left
				b.rect.left-=2*self.speed/FPS			
			#bullet out of screen --> destroy bullet
			if b.rect.right<0 or b.rect.left>=self.screenSize[0]:
				b=self.delObject(b,[self.bullets])
				#print 'bullet out of screen'
				continue
			
			#bullet hits cell of same color --> destroy bullet
			for cell in self.lanes[b.lane]:
				if pygame.sprite.collide_rect(b,cell):
					bulleHit=self.bulletOnCell(b,cell)
					if bulleHit:
						b=self.delObject(b,[self.bullets])
						break
			if bulleHit:
				continue
				
			
				
			#your bullet hits enemy --> destroy bullet and subtract health to enemy
			if b.owner=='you':
				for e in self.enemies:
					if pygame.sprite.collide_rect(b,e):
						if b.lane==e.lane:
							killed=e.hit()
							if killed:
								e.death()
								self.enemyKilled(e)
							b=self.delObject(b,[self.bullets])
							bulleHit=True
							break
				if bulleHit:
					continue
			#enemy bullet hits you --> destory bullet and subtract health to you
			else:
				if pygame.sprite.collide_rect(b,self.you):
					if b.lane==self.you.lane:
						#self.you.setNextAnimation('hit',1,True)
						killed=self.you.hit()
						if killed:
							self.you.death()
							self.jumpToLevel(GAMEOVER)
							break	
						b=self.delObject(b,[self.bullets])
						bulleHit=True
						break
				if bulleHit:
					continue
		
		

		self.you.rect.left+=self.you.flyFWSpeed
		self.you.rect.top+=self.you.flyUPSpeed
		


		#self.speedtext.editText('flyUPSpeed='+str(self.you.flyUPSpeed)+' lane='+str(self.you.lane)+' centery='+str(self.you.rect.centery)+'/'+str(self.getYfromLane(self.you.lane))+', STATUS: '+str(self.you.status))
		return False

			

class LevelOne(MadColorScience):
	def __init__(self, nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers):
		MadColorScience.__init__(self, nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)
		self.name='Lev0: Tutorial'
		
		self.lanesTotalNumber=3
		self.lanesColors=[pygame.Color('Red'),pygame.Color('Yellow'),pygame.Color('Blue')]	
		self.JollyColor=pygame.Color('White')
		self.LoserColor=pygame.Color('Black')		
		self.lanesCenterYOfMiddleOne=self.screenSize[1]/2
		self.lanesAngleDegrees=5 #corsie oblique da implementare
		self.lanesHeight=self.screenSize[1]/8
		self.lanesGapBetween=self.lanesHeight/10
		self.cellsTotalNumber=32
		self.cellsWidth=self.screenSize[0]/8
		self.cellsGapBetween=2
		self.generateLanes()
		
		self.enemiesTotalNumber=10
		self.enemiesNotInCellRange=[0,6]
		self.enemiesColors=self.lanesColors
		self.enemiesAvailable=['stop','move','shot','color']
		self.tetrisPieces=self.generateTetrisPieces()
		self.generateEnemies(self.enemiesTotalNumber)		
		
		self.yourBulletColors=[pygame.Color('Red'),pygame.Color('Yellow'),pygame.Color('Blue')]
		self.yourSize=(self.cellsWidth*2,self.cellsWidth*2)
		self.yourStartingLane=self.lanesTotalNumber/2
		self.yourStartingCell=1
		self.yourCenterPosition=self.getPositionFromLaneCell(self.yourStartingLane,self.yourStartingCell)
		self.you=self.addObject(customObjects.You(self.animations['obj_pheega_color.png'],self.yourStartingLane,self.yourSize,self.yourCenterPosition))
		self.you.lives=self.inventory['lives']
		#self.you.changeColorToAllFrames(pygame.Color('White'),self.lanesColors[self.you.laser])
		

class LevelTwo(MadColorScience):
	def __init__(self, nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers):
		MadColorScience.__init__(self, nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)
		self.name='Lev2: Subtractive color'
		
		self.lanesTotalNumber=6
		self.lanesColors=[pygame.Color('Red'),pygame.Color('Orange'),pygame.Color('Yellow'),pygame.Color('Green'),pygame.Color('Blue'),pygame.Color('Purple')]		
		self.JollyColor=pygame.Color('White')
		self.LoserColor=pygame.Color('Black')
		self.lanesCenterYOfMiddleOne=self.screenSize[1]/2
		self.lanesAngleDegrees=5 #corsie oblique da implementare
		self.lanesHeight=self.screenSize[1]/12
		self.lanesGapBetween=self.lanesHeight/10
		self.cellsTotalNumber=32
		self.cellsWidth=self.screenSize[0]/8
		self.cellsGapBetween=2
		self.generateLanes()

		self.enemiesTotalNumber=20
		self.enemiesNotInCellRange=[0,6]
		self.enemiesColors=self.lanesColors+[self.LoserColor]
		self.enemiesAvailable=['stop','move','shot','color','move+shot','move+color','shot+color'] #level>1
		self.tetrisPieces=self.generateTetrisPieces()
		self.generateEnemies(self.enemiesTotalNumber)
		
		self.yourBulletColors=[pygame.Color('Red'),pygame.Color('Yellow'),pygame.Color('Blue')]
		self.yourSize=(self.cellsWidth*2,self.cellsWidth*2)
		self.yourStartingLane=self.lanesTotalNumber/2
		self.yourStartingCell=1
		self.yourCenterPosition=self.getPositionFromLaneCell(self.yourStartingLane,self.yourStartingCell)
		self.you=self.addObject(customObjects.You(self.animations['obj_pheega_color.png'],self.yourStartingLane,self.yourSize,self.yourCenterPosition))		
		self.you.lives=self.inventory['lives']
		#self.you.changeColorToAllFrames(pygame.Color('White'),self.lanesColors[self.you.laser])
		
	def bulletOnCell(self, bullet, cell):
		#this do not consider alpha
		bulletColor=pygame.Color(bullet.color.r,bullet.color.g,bullet.color.b,255)
		cellColor=pygame.Color(cell.color.r,cell.color.g,cell.color.b,255)
		if cellColor==self.JollyColor or\
				bulletColor==self.JollyColor:
			return False
		elif cellColor==self.LoserColor or\
				abs(self.lanesColors.index(bulletColor)-self.lanesColors.index(cellColor)) in [0,1,self.lanesTotalNumber-1]:
			return True
		else:
			return False		

class LevelThree(MadColorScience):
	def __init__(self, nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers):
		MadColorScience.__init__(self, nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)
		self.name='Lev3: Additive color'
		
		self.lanesTotalNumber=6
		self.lanesColors=[pygame.Color('Red'),pygame.Color('Purple'),pygame.Color('Blue'),pygame.Color('Cyan'),pygame.Color('Green'),pygame.Color('Yellow')]		
		self.JollyColor=pygame.Color('Black')	
		self.LoserColor=pygame.Color('White')	
		self.lanesCenterYOfMiddleOne=self.screenSize[1]/2
		self.lanesAngleDegrees=5 #corsie oblique da implementare
		self.lanesHeight=self.screenSize[1]/12
		self.lanesGapBetween=self.lanesHeight/10
		self.cellsTotalNumber=32
		self.cellsWidth=self.screenSize[0]/8
		self.cellsGapBetween=2
		self.generateLanes()

		self.enemiesTotalNumber=20
		self.enemiesNotInCellRange=[0,6]
		self.enemiesColors=self.lanesColors+[self.LoserColor]
		self.enemiesAvailable=['stop','move','shot','color','move+shot','move+color','shot+color'] #level>1
		self.tetrisPieces=self.generateTetrisPieces()
		self.generateEnemies(self.enemiesTotalNumber)
		self.generateEnemies(1, 'boss')
		
		self.yourBulletColors=[pygame.Color('Red'),pygame.Color('Blue'),pygame.Color('Green')]
		self.yourSize=(self.cellsWidth*2,self.cellsWidth*2)
		self.yourStartingLane=self.lanesTotalNumber/2
		self.yourStartingCell=1
		self.yourCenterPosition=self.getPositionFromLaneCell(self.yourStartingLane,self.yourStartingCell)
		self.you=self.addObject(customObjects.You(self.animations['obj_pheega_color.png'],self.yourStartingLane,self.yourSize,self.yourCenterPosition))		
		self.you.lives=self.inventory['lives']
		#self.you.changeColorToAllFrames(pygame.Color('White'),self.lanesColors[self.you.laser])
		
	def bulletOnCell(self, bullet, cell):
		#this do not consider alpha
		bulletColor=pygame.Color(bullet.color.r,bullet.color.g,bullet.color.b,255)
		cellColor=pygame.Color(cell.color.r,cell.color.g,cell.color.b,255)
		if cellColor==self.JollyColor or\
				bulletColor==self.JollyColor:
			return False
		elif cellColor==self.LoserColor or\
				abs(self.lanesColors.index(bulletColor)-self.lanesColors.index(cellColor)) in [0,1,self.lanesTotalNumber-1]:
			return True
		else:
			return False				

class LevelFour(MadColorScience):
	def __init__(self, nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers):
		MadColorScience.__init__(self, nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)
		self.name='Bonus: Arctic madness'
		self.randomizeColors=True
		
	def initializeWithRandomColors(self):	
		self.lanesTotalNumber=6
		#self.lanesColors=[]
		#self.JollyColor=None
		#self.LoserColor=None
		self.lanesCenterYOfMiddleOne=self.screenSize[1]/2
		self.lanesAngleDegrees=5 #corsie oblique da implementare
		self.lanesHeight=self.screenSize[1]/12
		self.lanesGapBetween=self.lanesHeight/10
		self.cellsTotalNumber=32
		self.cellsWidth=self.screenSize[0]/8
		self.cellsGapBetween=2
		self.generateLanes()

		self.enemiesTotalNumber=50
		self.enemiesNotInCellRange=[0,6]
		self.enemiesColors=self.lanesColors+[self.LoserColor]
		self.enemiesAvailable=['stop','move','shot','color','move+shot','move+color','shot+color'] #level>1
		self.tetrisPieces=self.generateTetrisPieces()
		self.generateEnemies(self.enemiesTotalNumber)
		self.generateEnemies(1, 'boss')
		
		self.yourBulletColors=[pygame.Color('Red'),pygame.Color('Blue'),pygame.Color('Green')]
		self.yourSize=(self.cellsWidth*2,self.cellsWidth*2)
		self.yourStartingLane=self.lanesTotalNumber/2
		self.yourStartingCell=1
		self.yourCenterPosition=self.getPositionFromLaneCell(self.yourStartingLane,self.yourStartingCell)
		self.you=self.addObject(customObjects.You(self.animations['obj_pheega_color.png'],self.yourStartingLane,self.yourSize,self.yourCenterPosition))		
		self.you.lives=self.inventory['lives']
		

	def bulletOnCell(self, bullet, cell):
		#this do not consider alpha
		bulletColor=pygame.Color(bullet.color.r,bullet.color.g,bullet.color.b,255)
		cellColor=pygame.Color(cell.color.r,cell.color.g,cell.color.b,255)
		try:
			if cellColor==self.JollyColor or\
					bulletColor==self.JollyColor:
				return False
			elif cellColor==self.LoserColor or\
					abs(self.lanesColors.index(bulletColor)-self.lanesColors.index(cellColor)) in [0,1,self.lanesTotalNumber-1]:
				return True
			else:
				return False
		except ValueError:
			return False

class Title(myLevels.Title):
	def __init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory, menuText=[], textColors=[pygame.Color('Salmon'),pygame.Color('Red')], textPos=[0,0], backgroundImageName='screen_title.jpg'):

		myLevels.Title.__init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory, menuText, textColors, textPos, backgroundImageName)
		self.madColors=inventory['madColors']
		self.inventory['lives']=20
		'''
		for row in range(len(menuText)):
			textPos=(screenSize[0]*25/32,screenSize[1]*2/8)
			button=self.addObject(customObjects.Button(animations['obj_button.png'],(180,60),(screenSize[0]*28/32,screenSize[1]*3/16*(2+row))))		
		'''
		
	def checkKeyPressure(self,keyEvent):
		if keyEvent.type == KEYDOWN:
			if keyEvent.key == K_UP:
				self.menuValue=(self.menuValue-1)%len(self.menuText)
				self.updateMenu()
				return True
			elif keyEvent.key == K_DOWN:
				self.menuValue=(self.menuValue+1)%len(self.menuText)		
				self.updateMenu()
				return True
			elif keyEvent.key == K_RETURN:
				if self.menuText[self.menuValue]=='play':
					self.jumpToLevel(GAMEINTRO)
				elif self.menuText[self.menuValue]=='???':
					pass
				elif self.menuText[self.menuValue]=='Arctic madness':
					if self.madColors:
						self.jumpToLevel(MADCOLORSINTRO)
				elif self.menuText[self.menuValue][:5]=='audio':
					pygame.event.post(pygame.event.Event(TOGGLE_AUDIO))
					self.menuText[self.menuValue]='audio '+(not self.musicOn)*'on'+(self.musicOn)*'off'
					self.menu[self.menuValue].editText(self.menuText[self.menuValue])
				elif self.menuText[self.menuValue]=='credits':
					self.jumpToLevel(CREDITS)
				elif self.menuText[self.menuValue]=='quit':
					sys.exit('quit from menu')					
				else:
					print 'ERROR',self.menuValue,self.menuText[self.menuValue]
					self.menuValue=0
				self.updateMenu()
				return True
			
class Credits(myLevels.Credits):
	def __init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory, textColors=[pygame.Color('Salmon'),pygame.Color('Red')], backgroundImageName='screen_credits.png', sortedLayers=[]):
		myLevels.Credits.__init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory, textColors, backgroundImageName, sortedLayers)
		
	def checkKeyPressure(self,keyEvent):
		if keyEvent.type == KEYDOWN:
			self.jumpToLevel(TITLESCREEN)
			return True

