'''Game main module.

'''
import sys, pygame, random, math, os, time, traceback
import myDataloader, myResolution, myFrames, myLevels, myObjects
import PW14_levels as customLevels
import PW14_objects as customObjects
from myConstants import *

from pygame.locals import *
pygame.init()
pygame.font.init()


clock = pygame.time.Clock()
debugOn=False
musicOn=False

screen=myResolution.getResolution(DEFAULT_SCREEN_SIZE)
screenRatio=[float(screen.get_width())/DEFAULT_SCREEN_SIZE[0],float(screen.get_height())/DEFAULT_SCREEN_SIZE[1]]
pygame.display.set_caption('test')
myFrames.initialize()

try:
	myDataloader.load_music('PT9P_theme.mp3')
except pygame.error: 
	print '=============================='
	print 'WARNING: music file not found.'
	print '=============================='


	
	
########################################
def createLevel(nextLevel,animations,inventory):
	screenSize=(screen.get_width(),screen.get_height())
	sortedLayers=[customObjects.FarBackGround,customObjects.CloseBackGround,customObjects.ColorSquareSection,customObjects.Enemy,customObjects.You,customObjects.Bullet,customObjects.PauseScreen,customObjects.Button,myObjects.GameText]
	#print nextLevel,inventory
	#splashscreen and credits		
	if nextLevel==SPLASHSCREEN:		
		currentLevel=myLevels.Splash(nextLevel,screenSize,'screen_forkedtaillogo.png',[pygame.Color('Red')], debugOn, musicOn, inventory, sortedLayers)
	else:
		#start music
		if not pygame.mixer.music.get_busy() and musicOn:
				pygame.mixer.music.play(-1)		  
		#choose level
		if nextLevel==GAMEINTRO:
			storyLength=7
			currentLevel=myLevels.Intro(nextLevel,screenSize,animations,storyLength,'intro', debugOn, musicOn, inventory)
		elif nextLevel==LEVEL0:
			currentLevel=customLevels.LevelOne(nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)
		elif nextLevel==INTERVAL:
			storyLength=3
			currentLevel=myLevels.Intro(nextLevel,screenSize,animations,storyLength,'interval', debugOn, musicOn, inventory)			
		elif nextLevel==LEVEL2:
			currentLevel=customLevels.LevelTwo(nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)	
		elif nextLevel==LEVEL3:
			currentLevel=customLevels.LevelThree(nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)		
		elif nextLevel==MADCOLORSINTRO:
			storyLength=2
			currentLevel=myLevels.Intro(nextLevel,screenSize,animations,storyLength,'madintro', debugOn, musicOn, inventory)			
		elif nextLevel==MADCOLORS:
			currentLevel=customLevels.LevelFour(nextLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)
		elif nextLevel==MADYOUWIN:
			storyLength=2
			currentLevel=myLevels.Intro(nextLevel,screenSize,animations,storyLength,'youwin', debugOn, musicOn, inventory)				
		elif nextLevel==GAMEOVER:
			storyLength=1
			currentLevel=myLevels.Intro(nextLevel,screenSize,animations,storyLength,'gameover', debugOn, musicOn, inventory)			
		elif nextLevel==YOUWIN:
			storyLength=2
			currentLevel=myLevels.Intro(nextLevel,screenSize,animations,storyLength,'youwin', debugOn, musicOn, inventory)		
		elif nextLevel==CREDITS:
			storyLength=1
			currentLevel=myLevels.Intro(nextLevel,screenSize,animations,storyLength,'credits', debugOn, musicOn, inventory)	
		else:
			#includes nextLevel==TITLESCREEN:
			print 'createLevel() - nextLevel=',nextLevel
			menuText=['play',(not inventory['madColors'])*'???'+(inventory['madColors'])*'Arctic madness','audio '+(not musicOn)*'on'+(musicOn)*'off','credits','quit']
			textColors=[pygame.Color(120,200,240,255),pygame.Color(214,205,154,255)]
			textPos=(screenSize[0]*26/32,screenSize[1]*2/8)
			backgroundImageName='screen_title1.png'
			currentLevel=customLevels.Title(nextLevel,screenSize,animations, debugOn, musicOn, inventory,menuText, textColors, textPos, backgroundImageName)		
			#sys.exit('Level '+str(nextLevel)+' does not exist')	
			#sys.exit('END OF GAME')	
	return currentLevel	


def mainLoop(currentLevel):  
	screen.fill([0,0,0])
	screen.blit(currentLevel.backgroundImage, (0,0))	
	#level loop
	while currentLevel.levelCompleted==False:
		try:
			clock.tick(FPS) 
			#manage key events
			for event in pygame.event.get():
				currentLevel.eventsManagement(event)				
			#update objects
			currentLevel.update()

			#draw background
			'''#scrolling screen
			if currentLevel.scrolled:
				screen.blit(currentLevel.backgroundImage, (0,0))
				currentLevel.scrolled=False
			'''
			screen.blit(currentLevel.backgroundImage, (0,0))
			currentLevel.levelObjects.clear(screen, currentLevel.backgroundImage)
			
			#draw objects
			'''#scrolling screen
			pygame.display.update(currentLevel.levelObjects.draw(screen, currentLevel.scrollDelta))
			'''
			pygame.display.update(currentLevel.levelObjects.draw(screen))
			
			#draw FPS
			FPSfont=pygame.font.Font(myDataloader.filepath(TITLECUSTOMFONT), 20)
			#screen.blit(FPSfont.render('FPS '+str(round(clock.get_fps(),1)), True, pygame.Color('Black'), pygame.Color('White')).convert_alpha(), (0,0))
			if currentLevel.currentLevel in (LEVEL0,LEVEL2,LEVEL3,MADCOLORS):
				screen.blit(FPSfont.render('FPS '+str(round(clock.get_fps(),1))+' - '+currentLevel.name+' - Esc to Pause/Help', True, pygame.Color('Black'), pygame.Color('White')).convert_alpha(), (0,screen.get_height()-20))	
			
			#refresh screen
			pygame.display.flip()
		except:
			raise
			FATALERROR=True
			errorfile=open('errors.log', 'a')
			errorfile.write('-----FATAL ERRROR-----'+'\n'+str(time.localtime())+'\n')
			traceback.print_exc(file=errorfile)
			print '-------------------------------------\nFATAL ERRROR. Please check errors.log\n-------------------------------------'
			break
	return currentLevel



def resetGame():
	inventory={}
	inventory['lives']=10
	inventory['points']=0
	inventory['madColors']=False
	return inventory

	
	
def main():
	global debugOn
	global musicOn	
	currentLevel=None
	nextLevel=SPLASHSCREEN
	FATALERROR=False
		
	#animations stores images to be used in every level
	animations={}

	#inventory stores data across different levels (e.g. lives, point, etc...)
	inventory=resetGame()
	
	while not FATALERROR:
		#create level
		currentLevel=createLevel(nextLevel,animations,inventory)
		
		#main loop
		try:
			currentLevel=mainLoop(currentLevel)
		except:
			print traceback.format_exc()
			traceback.print_exc(file=open("error.log","a"))
			sys.exit(1)
		
		#load animation with splashscreen
		if currentLevel.currentLevel==SPLASHSCREEN:
			animations=currentLevel.animations
		
		#save current state for next level
		try:
			#print 'was',currentLevel.inventory
			inventory=currentLevel.inventory
			debugOn=currentLevel.debugOn
		except:
			print 'ERROR - resetting game'
			inventory=resetGame()
		
		#change level
		nextLevel=currentLevel.nextLevel
