import sys, time, pygame
from pygame.locals import *

import myDataloader, myFrames, myObjects
from myConstants import *


class Level():
	def __init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers=[]):
		self.currentLevel=currentLevel
		self.nextLevel=self.currentLevel+1
		self.screenSize=screenSize
		self.screenRatio=[float(self.screenSize[0])/DEFAULT_SCREEN_SIZE[0],float(self.screenSize[1])/DEFAULT_SCREEN_SIZE[1]]		
		self.animations=animations
		self.debugOn=debugOn
		self.musicOn=musicOn
		self.inventory=inventory
		self.listOfMissingFiles=[]

		self.levelCompleted=False
		self.levelPaused=False
		
		self.sortLayers(sortedLayers)
		self.levelObjects=myObjects.MyLayeredUpdates()

		self.initialSetup()

		#cleans key pressure at level start
		pygame.event.clear()

   
	def initialSetup(self):
		pass

	def setBackgroundOld(self, filename):
		tmpBackgroundImage=self.getAnimations(filename)['stop'][0]
		imageOriginalSize=tmpBackgroundImage.get_size()
		sizeRatio=min(float(self.screenSize[0])/imageOriginalSize[0],float(self.screenSize[1])/imageOriginalSize[1])
		self.backgroundImage=pygame.transform.scale(tmpBackgroundImage,[int(sizeRatio*imageOriginalSize[0]), int(sizeRatio*imageOriginalSize[1])])
		
	def setBackground(self, filename):
		self.backgroundImage=pygame.Surface(self.screenSize,pygame.SRCALPHA, 32).convert_alpha()
		self.backgroundImage.fill(pygame.Color(137,251,137))
		tmpBackgroundImage=self.getAnimations(filename)['stop'][0]
		imageOriginalSize=tmpBackgroundImage.get_size()
		sizeRatio=min(float(self.screenSize[0])/imageOriginalSize[0],float(self.screenSize[1])/imageOriginalSize[1])
		blitme=pygame.transform.scale(tmpBackgroundImage,[int(sizeRatio*imageOriginalSize[0]), int(sizeRatio*imageOriginalSize[1])])
		self.backgroundImage.blit(blitme,(0,0))
		
	def getAnimations(self,name):
		try:
			return self.animations[name]
		except KeyError:
			if name not in self.listOfMissingFiles:
				self.listOfMissingFiles.append(name)
				print '!!! MISSING FILE:',name,'!!!'
			return self.animations['miss_image.png']
		
	def changeStatus(self,status=None):
		if status<>None:
			self.status=status
		else:
			self.status+=1
		self.timeStart=time.time()

	#order of the objects layer, from back to front (last object will be always visible)
	def sortLayers(self, sortedLayers):
		self.layerOrder=sortedLayers

	#adds specified object to levelObjects list setting proper layerOrder
	def addObject(self, newObj):
		spriteLayer=0
		try:
			spriteLayer=self.layerOrder.index(newObj.__class__)
		except ValueError:
			try:
				for parent in newObj.__class__.__bases__:
					spriteLayer=self.layerOrder.index(parent)
					#print spriteLayer
			except ValueError:
				spriteLayer=0
			
		self.levelObjects.add(newObj, layer=spriteLayer)
		return newObj 
	
	#remove specified object from levelObjects
	def delObject(self, delObj, lists=[]):
		try:
			#print delObj,'object deleted from self.levelObjects'
			self.levelObjects.remove(delObj)
		except:
			print '######## WTF? ###########'
		try:
			for l in lists:
			   l.remove(delObj)
			   #print 'object deleted from',l
		except:
			print '######## WTF? ###########'
		return None

	def eventsManagement(self,event):
		if event.type == DEBUG_MODE: #debug on/off
			self.debugOn = not self.debugOn			
		elif event.type == TOGGLE_AUDIO: #audio on/off
			if self.musicOn:
				pygame.mixer.music.stop()
			else:
				pygame.mixer.music.play()
			self.musicOn = not self.musicOn
		
		keyPressureCaught=False
		if event.type in (KEYDOWN,KEYUP,MOUSEMOTION,MOUSEBUTTONUP,MOUSEBUTTONDOWN): #keys
			keyPressureCaught=self.checkKeyPressure(event)
		if not keyPressureCaught:
			if event.type == KEYDOWN:
				if event.key == K_ESCAPE: #quit
					sys.exit('Pressed ESC')
				if event.key == K_BACKSPACE: #restart level
					self.nextLevel=self.currentLevel
					self.setLevelCompleted()
				if event.key == K_PAGEUP:
					self.jumpToLevel(self.currentLevel-1)
				if event.key == K_PAGEDOWN:
					self.jumpToLevel(self.currentLevel+1)

	
		   
	def checkKeyPressure(self,keyEvent):
		#method to be overwritten
		return False
				
	def update(self):
		self.levelObjects.update()
		self.customUpdate()
	
	def customUpdate(self):
		pass	

	def setLevelCompleted(self):
		self.levelCompleted=True
	
	def jumpToLevel(self, nextLevelNum):
		self.nextLevel=nextLevelNum
		self.setLevelCompleted()

	def debugPrint(self, text):
		if self.debugOn:
			print str(text)


class Splash(Level):
	def __init__(self, currentLevel, screenSize, backgroundImageName, textColors, debugOn, musicOn, inventory, sortedLayers=[]):
		self.screenSize=screenSize
		self.backgroundImage=myDataloader.load_image(backgroundImageName, self.screenSize)
		self.firstLoop=True
		self.imageFileNames=myDataloader.getImageFileNames()
		Level.__init__(self, currentLevel, screenSize, {}, debugOn, musicOn, inventory, sortedLayers)
		self.textColors=textColors
		self.createInfoText()

	def createInfoText(self):
		textSize=self.screenSize[0]/40
		self.infoText=self.addObject(myObjects.GameText('',textSize,[self.screenSize[0]/16,self.screenSize[1]*11/12],self.textColors[0]))
		
	def splitImageIntoFrames(self,fullImage,framesInaRowCol):
		framesInaRow,framesInaCol=framesInaRowCol
		frame_width = fullImage.get_width()/framesInaRow
		frame_height = fullImage.get_height()/framesInaCol
		
		animationTuple=[]
		for row in range(framesInaCol):
			for col in range(framesInaRow):
				singleFrame=fullImage.subsurface((col*frame_width,row*frame_height,frame_width,frame_height))
				animationTuple.append(singleFrame)
		return animationTuple
	
	def assignFramesToAction(self,imgName,animationTuple):
		actionAnimationDict={}
		try:
			for action in myFrames.myFrames[imgName].keys():
				actionAnimationTuple=[]
				if action not in ['framesInaRow','framesInaCol']:
					for f in myFrames.getFrames(imgName, action):
						actionAnimationTuple.append(animationTuple[f-1])
					actionAnimationDict[action]=actionAnimationTuple
		except IndexError:
			print imgName,action,'has ',len(animationTuple),'(',myFrames.myFrames[imgName]['framesInaRow'],'x',myFrames.myFrames[imgName]['framesInaCol'],') frames. frame #',(f-1),'has been assigned to action',action
			raise IndexError
		return actionAnimationDict
		
	def customUpdate(self):
		if self.firstLoop==True: #first loop does nothing, just draw background
			self.firstLoop=False
		else:
			if len(self.imageFileNames)>0:
				imgName=self.imageFileNames.pop()
				print 'loading', imgName
				self.infoText.editText(imgName)
				self.infoText.setColor(self.textColors[len(self.imageFileNames)%len(self.textColors)])
				
				if imgName in myFrames.myFrames.keys():
					img=myDataloader.load_image(imgName)
					animationTuple=self.splitImageIntoFrames(img,myFrames.getRowCol(imgName))
					self.animations[imgName]=self.assignFramesToAction(imgName,animationTuple)
				else:
					img=myDataloader.load_image(imgName)
					self.animations[imgName]={}
					self.animations[imgName]['stop']=[img]
				self.animations[imgName]['size']=self.animations[imgName]['stop'][0].get_size()			   
			else:
				self.jumpToLevel(TITLESCREEN)
			

class Title(Level):
	def __init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory, menuText=[], textColors=[pygame.Color('Salmon'),pygame.Color('Red')], textPos=[0,0], backgroundImageName='screen_title.jpg'):
		Level.__init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory)	
		self.setBackground(backgroundImageName)
		self.createMenu(menuText,textColors, textPos)

	def createMenu(self, menuText, textColors, textPos):
		self.textColors=textColors 
		self.menuText=menuText
		self.menuValue=0
		self.menu=[]
		textSize=self.screenSize[0]/20
		for mt in range(len(self.menuText)):
			self.menu.append(self.addObject(myObjects.GameText(self.menuText[mt],textSize,[textPos[0],textPos[1]+textSize*mt],self.textColors[0])))
		self.menu[self.menuValue].setColor(self.textColors[1])		
		self.updateMenu()

	def updateMenu(self):
		#update color
		for m in range(len(self.menu)):
			if m==self.menuValue:
				self.menu[m].setColor(self.textColors[1])
			else:
				self.menu[m].setColor(self.textColors[0])		 
		
	
	def checkKeyPressure(self,keyEvent):
		if keyEvent.type == KEYDOWN:
			if keyEvent.key == K_UP:
				self.menuValue=(self.menuValue-1)%len(self.menuText)
				self.updateMenu()
				return True
			elif keyEvent.key == K_DOWN:
				self.menuValue=(self.menuValue+1)%len(self.menuText)		
				self.updateMenu()
				return True
			elif keyEvent.key == K_RETURN:
				self.setLevelCompleted()
				self.updateMenu()
				return True		

class Credits(Level):
	def __init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory, textColors=[pygame.Color('Salmon'),pygame.Color('Red')], backgroundImageName='screen_credits.png', sortedLayers=[]):
		Level.__init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)
		self.setBackground(backgroundImageName)
		self.createInfoText(textColors)

	def createInfoText(self, textColors):
		self.textColors=textColors
		textSize=self.screenSize[0]/20
		self.infoText=self.addObject(myObjects.GameText('click to continue...',textSize,[self.screenSize[0]/16,self.screenSize[1]*11/12],self.textColors[0]))
		
	def checkKeyPressure(self,keyEvent):
		if keyEvent.type == KEYDOWN and keyEvent.key <> K_ESCAPE:
			self.setLevelCompleted()
			return True
		if keyEvent.type == MOUSEBUTTONDOWN:
			self.setLevelCompleted()
			return True


class Intro(Level):
	def __init__(self, currentLevel, screenSize, animations, storyLength, imageName, debugOn, musicOn, inventory):
		Level.__init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory)
		self.storyLength=storyLength
		self.storyIndex=1
		self.imageName=imageName
		self.setBackground('screen_'+self.imageName+str(self.storyIndex)+'.png')
  
	def checkKeyPressure(self,keyEvent):
		if keyEvent.type == KEYDOWN and keyEvent.key <> K_ESCAPE:
			if self.storyIndex<self.storyLength:
				self.storyIndex+=1
				self.setBackground('screen_'+self.imageName+str(self.storyIndex)+'.png')
			else:
				self.setLevelCompleted()
			return True



