from myConstants import *
from pygame.locals import *
pygame.init()

class GameText(pygame.sprite.Sprite):  
	def __init__(self, text, size, topleft, color):
		pygame.sprite.Sprite.__init__(self)
		self.text=text
		self.size=size
		self.initialtopleft=topleft
		self.color=color
		self.font=pygame.font.Font(myDataloader.filepath(CUSTOMFONT), self.size)
		self.image=self.font.render(self.text, True, self.color).convert_alpha()
		self.rect=self.image.get_rect()
		self.rect.topleft=topleft
		self.scrollDelta=[0,0]
	
	def editText(self,newtext):
		self.text=str(newtext)
		self.update()
		
	def move(self,newtopleft):
		self.rect.topleft=newtopleft
		
	def resize(self,newsize):
		self.size=newsize
		self.font=pygame.font.Font(myDataloader.filepath(CUSTOMFONT), self.size)
		self.update()
		
	def setColor(self,newcolor):
		self.color=newcolor
		self.update() 
	
	def update(self):
		self.image=self.font.render(self.text, True, self.color).convert_alpha()	 
	   

class LevelTimer(GameText):
	def __init__(self, size, topleft, color):
		GameText.__init__(self, '', size, topleft, color)
		self.counter=0

	def update(self):
		self.counter+=1
		self.text=str((self.counter/FPS)/60).rjust(2,'0')+':'+str((self.counter/FPS)%60).rjust(2,'0')+':'+str(self.counter%FPS).rjust(2,'0')
		self.image=self.font.render(self.text, True, self.color).convert_alpha()


class MyLayeredUpdates(pygame.sprite.LayeredUpdates): 
	def __init__(self, *sprites, **kwargs):
		pygame.sprite.LayeredUpdates.__init__(self, *sprites, **kwargs)

	def draw(self, surface, scrollDelta=[0,0]):
		"""draw all sprites in the right order onto the passed surface.
		LayeredUpdates.draw(surface): return Rect_list
		"""
		spritedict = self.spritedict
		surface_blit = surface.blit
		dirty = self.lostsprites
		self.lostsprites = []
		dirty_append = dirty.append
		for spr in self.sprites():
			scrolledRect=pygame.Rect([spr.rect.left-scrollDelta[0],spr.rect.top-scrollDelta[1]],spr.rect.size)
			rec = spritedict[spr]			
			newrect = surface_blit(spr.image, scrolledRect)
			if rec is 0:
				dirty_append(newrect)
			else:
				if newrect.colliderect(rec):
					dirty_append(newrect.union(rec))
				else:
					dirty_append(newrect)
					dirty_append(rec)
			spritedict[spr] = newrect
		return dirty	
	
	
class GameObject(pygame.sprite.Sprite): 
	def __init__(self, images, size, centerPosition, rgb=[255,255,255]):
		pygame.sprite.Sprite.__init__(self)
		self.size=int(size[0]),int(size[1])
		self.topleft=[int(centerPosition[0]-size[0]/2),int(centerPosition[1]-size[1]/2)]

		self.images=self.createCopyOfImage(images)

		self.image=pygame.Surface((self.size))		 

		self.rect=self.image.get_rect()
		self.setPosition(centerPosition)
		#self.scrollDelta=[0,0]
		
		self.r,self.g,self.b=rgb
		self.animationTimer=0		
		self.lookingLeft=False
		self.baseAnimationName='stop'
		self.currentAnimationName=self.baseAnimationName
		self.currentAnimationRepeat=-1 #-1 forever
		self.currentAnimationIndex=0
		self.setNextAnimation(self.baseAnimationName,-1,False)
		
		self.loadFrame()
		#full mask
		self.mask=pygame.mask.from_surface(self.image) 
		#collision mask (only when image has alpha levels)
		#self.collision_mask=pygame.mask.from_surface(self.image, 253)

	def createCopyOfImage(self,images):
		return images

			
	'''
	# you can use alpha levels to customize collisions
	def getRect4Collisions(self):
		rectSize=10
		return pygame.Rect(self.rect.centerx-rectSize/2,self.rect.centery-rectSize/2,rectSize+1,rectSize+1)
	'''

	def setTopLeft(self,topleft):
		self.rect.topleft=topleft

	def getTopLeft(self):
		return self.rect.topleft
   
	def setPosition(self, center):
		self.rect.center=center
	
	def getPosition(self):
		return self.rect.center
	
	def getSize(self):
		return self.size
	
	def setSize(self, newSize):
		oldCenter=self.rect.center
		self.size=newSize
		self.image=pygame.Surface((self.size))		 
		self.rect=self.image.get_rect()
		self.loadFrame()
		self.setPosition(oldCenter)
		
	def getAllFrames(self):
		frames=[]
		for k in self.images.keys():
			if k<>'size':
				for i in self.images[k]:
					frames.append(i)
		return frames

	def changeColorToAllFrames(self,fromColor,toColor):
		for i in self.getAllFrames():
			i = pygame.PixelArray(i)
			tolerance=0.1
			i.replace(fromColor, toColor, tolerance)

	
	def move(self):
		pass
				
	def loadFrame(self):
		try:
			self.image=pygame.transform.scale(self.images[self.currentAnimationName][self.currentAnimationIndex], self.size)
			if self.lookingLeft:
				self.image=pygame.transform.flip(self.image, True, False)
			self.image=self.get_alpha_surface(self.image, 255, self.r, self.g, self.b)
		except:
			raise			
		
	def changeAnimation(self):
		self.currentAnimationName=self.nextAnimationName
		self.currentAnimationRepeat=self.nextAnimationRepeat
		self.currentAnimationIndex=0
		self.nextAnimationName=self.baseAnimationName
		self.nextAnimationRepeat=-1
		self.nextAnimationForce=False
		
	def changeFrame(self):
		numOfAnim=len(self.images[self.currentAnimationName])
		self.currentAnimationIndex=(self.currentAnimationIndex+1)%numOfAnim
		
	def setNextAnimation(self,name,repeat=-1,force=False):
		self.nextAnimationName=name
		self.nextAnimationRepeat=repeat
		self.nextAnimationForce=force
	
	def updateAnimation(self):
		if self.nextAnimationForce:
			self.changeAnimation()
			self.nextAnimationForce=False
		else:
			self.changeFrame()
			if self.currentAnimationIndex>=len(self.images[self.currentAnimationName]):
				self.currentAnimationRepeat-=1
				if self.currentAnimationRepeat<=0:
					self.changeAnimation()
		self.loadFrame()
		
	def get_alpha_surface(self, surf, alpha=128, red=128, green=128, blue=128, mode=pygame.BLEND_RGBA_MULT):
		"""returns a copy of a surface object with user-defined 
		   values for red, green, blue and alpha. 
		   Values from 0-255. 
		   thanks to Claudio Canepa <ccanepacc@gmail.com>
		   for this function."""
		tmp = pygame.Surface(surf.get_size(), pygame.SRCALPHA, 32)
		tmp.fill( (red,green,blue,alpha) )
		tmp.blit(surf, (0,0), surf.get_rect(), mode)
		return tmp		
					
	def update(self):
		self.animationTimer=(self.animationTimer+1)%FPS_ANIMATIONS
		if self.animationTimer==0:
			self.updateAnimation()
		self.updatePosition()
		
	def updatePosition(self):
		pass	
		
class GameObjectReusable(GameObject): 
	def __init__(self, images, size, centerPosition, rgb=[255,255,255]):
		GameObject.__init__(self, images, size, centerPosition, rgb=[255,255,255])

	def createCopyOfImage(self,images):
		copiedImages={}
		for ik in images.keys():
			if ik<>'size':
				copiedImages[ik]=[]
				for s in images[ik]:
					copiedImages[ik].append(s.copy())
		copiedImages['size']=images['size']
		return copiedImages